USE [master]
GO
IF EXISTS(SELECT 1 FROM sys.databases WHERE name = 'DRTest')
BEGIN
	ALTER DATABASE DRTest SET SINGLE_USER WITH ROLLBACK IMMEDIATE
	DROP DATABASE DRTest
END
GO
CREATE DATABASE DRTest COLLATE SQL_Latin1_General_CP1_CI_AS
GO
USE [DRTest]
GO

/****** Object:  Table [dbo].[transactions]    Script Date: 10/04/2011 10:23:11 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

SET ANSI_PADDING ON
GO
IF OBJECT_ID('dbo.transactions') IS NOT NULL DROP TABLE dbo.transactions
CREATE TABLE [dbo].[transactions](
	[transaction_id] [int] NOT NULL PRIMARY KEY,
	[tran_data] [varchar](4000) NULL,
	[tran_number] [int] NULL,
	[created_date] [datetime] NOT NULL,
	[created_server] [varchar](100) NOT NULL
)
GO
SET ANSI_PADDING ON
GO
ALTER TABLE [dbo].[transactions] ADD  DEFAULT (getdate()) FOR [created_date]
GO
ALTER TABLE [dbo].[transactions] ADD  DEFAULT (CONVERT([varchar](100),serverproperty('ComputerNamePhysicalNetBIOS'),(0))) FOR [created_server]
GO

IF OBJECT_ID('dbo.current_id') IS NOT NULL DROP TABLE dbo.current_id
CREATE TABLE [dbo].[current_id](
	table_name sysname PRIMARY KEY,
	current_id_value int NOT NULL DEFAULT(0)
)

INSERT dbo.current_id (table_name) VALUES ('dbo.transactions')

/****** Object:  StoredProcedure [dbo].[transaction_insert]    Script Date: 10/04/2011 10:24:16 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('dbo.transaction_insert') IS NOT NULL DROP PROC dbo.transaction_insert
GO
CREATE PROC [dbo].[transaction_insert]
@DataLength int,
@UpdateNum bit,
@ID int OUTPUT,
@DateTime datetime OUTPUT,
@Server varchar(100) OUTPUT
AS
/*
DECLARE @ID int, @DateTime datetime, @Server varchar(100)
EXEC transaction_insert @DataLength = 50, @UpdateNum = 0, @ID = @ID OUTPUT, @DateTime = @DateTime OUTPUT, @Server = @Server OUTPUT
SELECT @ID, @DateTime, @Server
*/
BEGIN
BEGIN TRY
	DECLARE @Data varchar(4000) 
	SET @Data = ''

	WHILE LEN(@Data) < @DataLength
		SET @Data = @Data + SUBSTRING(CAST(NEWID() AS varchar(4000)),1,@DataLength - LEN(@Data))

	DECLARE @Output TABLE (dt datetime, Svr sysname)

	BEGIN TRAN

	UPDATE dbo.current_id SET @ID = current_id_value = current_id_value + 1 WHERE table_name = 'dbo.transactions'

	INSERT transactions (transaction_id, tran_data)
	OUTPUT inserted.created_date, inserted.created_server INTO @Output
	VALUES (@ID,@Data)

	COMMIT

	SELECT @DateTime = dt, @Server = Svr
	FROM @Output

	IF @UpdateNum = 1
		UPDATE transactions SET tran_number = @ID
END TRY
BEGIN CATCH
	IF @@TRANCOUNT > 0 ROLLBACK
	DECLARE @ErrMsg nvarchar(2000)
	PRINT '****************************Error Detected*************************************************************'
	PRINT 'Error Number: ' + ISNULL(CAST(ERROR_NUMBER() as varchar(1000)),'<null>')
	PRINT 'Error Severity: ' + ISNULL(CAST(ERROR_SEVERITY()as varchar(1000)),'<null>')
	PRINT 'Error State: ' + ISNULL(CAST(ERROR_STATE()as varchar(1000)),'<null>')
	PRINT 'Error Procedure: ' + ISNULL(CAST(ERROR_PROCEDURE() as varchar(1000)),'<null>')
	PRINT 'Error Line: ' + ISNULL(CAST(ERROR_LINE()as varchar(1000)),'<null>')
	PRINT 'Error Message: ' + ISNULL(CAST(ERROR_MESSAGE() as varchar(1000)),'<null>')
	PRINT ''
	PRINT ''
	SET @ErrMsg = 'Program: "' + ISNULL(PROGRAM_NAME(),'<null>') 
			+ '" Procedure: "' + ISNULL(ERROR_PROCEDURE(),'<null>') 
			+ '" Line: ' + ISNULL(CAST(ERROR_LINE() AS nvarchar),'<null>') 
			+ ' Error message: "' + ISNULL(ERROR_MESSAGE(),'<null>') + '"'
	
	RAISERROR(@ErrMsg,16,1)
END CATCH
END
GO

CREATE PROC [dbo].[transactions_read]
AS
BEGIN
	SELECT MAX(transaction_id) AS TranID, SERVERPROPERTY('ComputerNamePhysicalNetBIOS') AS ServerName FROM transactions
END
GO

IF NOT EXISTS(SELECT 1 FROM sys.server_principals WHERE name = 'DRTest_User' AND SID = 0x0221B680EA1A5441A5E679D91C2A9D2A)
BEGIN
	IF EXISTS(SELECT 1 FROM sys.server_principals WHERE name = 'DRTest_User' AND SID <> 0x0221B680EA1A5441A5E679D91C2A9D2A)
		DROP LOGIN [DRTest_User]
	CREATE LOGIN [DRTest_User] WITH PASSWORD = 'P@ssword01', SID = 0x0221B680EA1A5441A5E679D91C2A9D2A
END

IF NOT EXISTS(SELECT 1 FROM sys.database_principals WHERE name = 'DRTest_User')
	CREATE USER [DRTest_User] FOR LOGIN [DRTest_User]

GRANT EXEC ON dbo.transaction_insert TO [DRTest_User]
GO

-- Permissions

GRANT EXECUTE ON  [dbo].[transactions_read] TO [DRTest_User]
GO


/*
DECLARE @FileName varchar(255) 
SET @FileName = 'FUL_DRTest_' + REPLACE(REPLACE(CONVERT(char(19),GETDATE(),121),' ','_'),':','-') + '.BAK'
PRINT @FileName
BACKUP DATABASE DRTest TO DISK = @FileName
*/

