﻿Imports System.ComponentModel
Imports System.Data.SqlClient

Public Module bgDB
    WithEvents bw As New BackgroundWorker
    Dim lea As New LogEventArgs

    Sub New()
        bw.WorkerReportsProgress = True
    End Sub

    Function GetConnStr(inConnStr As String, Optional Read As Boolean = False)
        Dim cb As New SqlConnectionStringBuilder(inConnStr)
        cb.ConnectTimeout = 7
        cb.ApplicationName = "DRTest"
        If Read Then cb.ApplicationIntent = ApplicationIntent.ReadOnly
        Return cb.ConnectionString
    End Function

    Sub TestConnection(inConnStr As String)
        Dim args As New WorkerArgs
        args.ConnStr = GetConnStr(inConnStr)
        args.CmdText = "SELECT @@SERVERNAME AS Svr"
        args.CmdType = "TestConnection"
        If bw.IsBusy Then
            MsgBox("Worker busy")
        Else
            bw.RunWorkerAsync(args)
        End If
    End Sub

    Sub ClearAllPools()
        SqlConnection.ClearAllPools()
        MsgBox("Done")
    End Sub

    Sub RunCommand(inConnStr As String, LastID As String, LastServer As String, LastTranDate As String, DataLength As Integer, MassUpdate As Boolean)
        lea.LastID = LastID
        lea.DataLength = DataLength
        lea.LastTranServer = LastServer
        If IsDate(LastTranDate) Then
            lea.LastTranDate = LastTranDate
        Else
            lea.LastTranDate = DateTime.MinValue
        End If
        lea.MassUpdate = MassUpdate
        Dim args As New WorkerArgs
        args.ConnStr = GetConnStr(inConnStr)
        args.CmdText = "transaction_insert"
        args.CmdType = "RunCommand"
        If Not bw.IsBusy Then
            bw.RunWorkerAsync(args)
        End If
    End Sub

    Sub ReadNow(inConnStr As String)
        Dim args As New WorkerArgs
        args.ConnStr = GetConnStr(inConnStr, True)
        args.CmdText = "transactions_read"
        args.CmdType = "ReadNow"
        If bw.IsBusy Then
            MsgBox("Worker busy")
        Else
            bw.RunWorkerAsync(args)
        End If
    End Sub

    Sub DoWork(sender As Object, e As DoWorkEventArgs) Handles bw.DoWork
        lea.Message = Nothing
        lea.Messages = New List(Of String)
        Dim args As WorkerArgs = e.Argument
        Try
            Using cn As New SqlConnection(args.ConnStr)
                Using cmd As New SqlCommand(args.CmdText, cn)
                    'lea.Log = False
                    'lea.Message = "Opening Connection"
                    'bw.ReportProgress(0, lea)
                    cn.Open()
                    'lea.Message = "Connection Open"
                    'bw.ReportProgress(0, lea)

                    If args.CmdType <> "TestConnection" Then
                        cmd.CommandType = CommandType.StoredProcedure
                    End If

                    If args.CmdType = "RunCommand" Then
                        cmd.Parameters.AddWithValue("@DataLength", lea.DataLength)
                        cmd.Parameters.AddWithValue("@UpdateNum", lea.MassUpdate)
                        cmd.Parameters.Add("@ID", SqlDbType.Int).Direction = ParameterDirection.Output
                        cmd.Parameters.Add("@DateTime", SqlDbType.DateTime).Direction = ParameterDirection.Output
                        cmd.Parameters.Add("@Server", SqlDbType.VarChar, 100).Direction = ParameterDirection.Output
                        'lea.Message = "Executing command"
                        'bw.ReportProgress(0, lea)
                        cmd.ExecuteNonQuery()
                        'lea.Message = "Command complete"
                        'bw.ReportProgress(0, lea)

                        lea.Log = True

                        If lea.LastID <> "<none>" AndAlso cmd.Parameters("@ID").Value <> CInt(lea.LastID) + 1 Then
                            lea.Messages.Add("Data Loss: Last ID: " & lea.LastID & " Current ID: " & cmd.Parameters("@ID").Value)
                        End If
                        lea.LastID = cmd.Parameters("@ID").Value

                        If lea.LastTranDate <> DateTime.MinValue AndAlso DateDiff(DateInterval.Second, lea.LastTranDate, cmd.Parameters("@DateTime").Value) > 5 Then
                            lea.Messages.Add("Downtime detected: " & DateDiff(DateInterval.Second, lea.LastTranDate, cmd.Parameters("@DateTime").Value) & " seconds")
                        End If
                        lea.LastTranDate = cmd.Parameters("@DateTime").Value

                        If lea.LastTranServer <> "<none>" AndAlso cmd.Parameters("@Server").Value <> lea.LastTranServer Then
                            lea.Messages.Add("Server change: Last Server: " & lea.LastTranServer & " Current Server: " & cmd.Parameters("@Server").Value)
                        End If
                        lea.LastTranServer = cmd.Parameters("@Server").Value
                        lea.Type = "Inf"
                        lea.UpdateValues = True
                        'lea.Log = lea.Message <> "Command complete"
                        bw.ReportProgress(0, lea)
                    Else
                        Using rdr As SqlDataReader = cmd.ExecuteReader
                            While rdr.Read
                                If args.CmdType = "TestConnection" Then
                                    lea.Log = True
                                    lea.Message = "Connection Successful to " & rdr("Svr")
                                    lea.Type = "Inf"
                                    lea.UpdateValues = False
                                    bw.ReportProgress(0, lea)
                                Else
                                    lea.Log = True
                                    lea.Message = "Last ID entered is " & lea.LastID & " Read ID Value is " & rdr("TranID") & " command ran on server " & rdr("ServerName")
                                    lea.Type = "Inf"
                                    lea.UpdateValues = False
                                    bw.ReportProgress(0, lea)
                                End If
                            End While
                        End Using
                    End If
                End Using
            End Using
        Catch ex As Exception
            lea.Log = True
            lea.Message = ex.Message
            lea.Type = "Err"
            lea.UpdateValues = False
            bw.ReportProgress(0, lea)
        End Try
    End Sub
    Sub Progress(sender As Object, e As ProgressChangedEventArgs) Handles bw.ProgressChanged
        Dim lea As LogEventArgs = e.UserState
        If lea.UpdateValues Then
            frmDRTest.txtLastID.Text = lea.LastID
            frmDRTest.txtLastServer.Text = lea.LastTranServer
            frmDRTest.txtLastTranDate.Text = lea.LastTranDate
        End If
        If lea.Log Then
            If lea.Message <> Nothing Then
                frmDRTest.LogEvent(lea.Message, lea.Type)
            End If
            For Each Str As String In lea.Messages
                frmDRTest.LogEvent(Str, lea.Type)
            Next
            If frmDRTest.txtOutput.Text <> "Running" AndAlso lea.Message = Nothing AndAlso lea.Messages.Count = 0 Then
                frmDRTest.txtOutput.Text = "Running"
            End If
        Else
            frmDRTest.txtOutput.Text = lea.Message
        End If
    End Sub
End Module
Public Class LogEventArgs
    Public Log As Boolean
    Public UpdateValues As Boolean
    Public Message As String
    Public Messages As New List(Of String)
    Public Type As String
    Public LastID As String
    Public LastTranDate As DateTime
    Public LastTranServer As String
    Public DataLength As Integer
    Public MassUpdate As Boolean
End Class
Public Class WorkerArgs
    Public ConnStr As String
    Public CmdText As String
    Public CmdType As String
End Class
